@@ -13,6 +13,8 @@ import torchvision
 
 from einops import rearrange, reduce, repeat
 
+from dalle2_pytorch.attention import QueryAttnUpsample
+
 # constants
 
 MList = nn.ModuleList
@@ -243,6 +245,7 @@ class ResBlock(nn.Module):
     def forward(self, x):
         return self.net(x) + x
 
+# vqgan attention layer
 class VQGanAttention(nn.Module):
     def __init__(
         self,
@@ -375,7 +378,7 @@ class VQGanVAE(nn.Module):
 
         for layer_index, (dim_in, dim_out), layer_num_resnet_blocks, layer_use_attn in zip(range(layers), dim_pairs, num_resnet_blocks, use_attn):
             append(self.encoders, nn.Sequential(nn.Conv2d(dim_in, dim_out, 4, stride = 2, padding = 1), leaky_relu()))
-            prepend(self.decoders, nn.Sequential(nn.Upsample(scale_factor = 2, mode = 'bilinear', align_corners = False), nn.Conv2d(dim_out, dim_in, 3, padding = 1), leaky_relu()))
+            prepend(self.decoders, nn.Sequential(QueryAttnUpsample(dim_out), nn.Conv2d(dim_out, dim_in, 3, padding = 1), leaky_relu()))
 
             if layer_use_attn:
                 prepend(self.decoders, VQGanAttention(dim = dim_out, heads = attn_heads, dim_head = attn_dim_head, dropout = attn_dropout))
